/**
 * 
 */
package org.springblade.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.springblade.pool.BrowserContextObjectPool;
import org.springblade.pool.BrowserContextPooledObjectFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;

import com.microsoft.playwright.BrowserContext;

import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Configuration
@EnableConfigurationProperties(PlaywrightPoolProperties.class)
public class PoolAutoConfiguration {

	@Autowired
	private PlaywrightPoolProperties poolProperties;
	
	/**
	 * 跨域支持
	 * @return
	 */
	@Bean
	public CorsFilter corsFilter() {
		CorsConfiguration corsConfiguration = new CorsConfiguration();
		corsConfiguration.addAllowedOriginPattern("*");
		corsConfiguration.addAllowedHeader("*");
		corsConfiguration.addAllowedMethod("*");
		corsConfiguration.setAllowCredentials(true);

		UrlBasedCorsConfigurationSource urlBasedCorsConfigurationSource = new UrlBasedCorsConfigurationSource();
		urlBasedCorsConfigurationSource.registerCorsConfiguration("/**", corsConfiguration);
		return new CorsFilter(urlBasedCorsConfigurationSource);
	}

	@SuppressWarnings("resource")
	@Bean(destroyMethod = "close")
	protected BrowserContextObjectPool playwrightPool() {
		BrowserContextPooledObjectFactory playwrightFactory = new BrowserContextPooledObjectFactory();
		// 设置对象池的相关参数
		GenericObjectPoolConfig<BrowserContext> poolConfig = new GenericObjectPoolConfig<>();
		poolConfig.setMaxIdle(poolProperties.getSize());
		poolConfig.setMaxTotal(poolProperties.getSize());
		poolConfig.setMinIdle(poolProperties.getSize());
		poolConfig.setBlockWhenExhausted(true);
		poolConfig.setTestOnBorrow(true);
		poolConfig.setTestOnReturn(true);
		poolConfig.setTestWhileIdle(true);
		poolConfig.setTimeBetweenEvictionRunsMillis(1000 * 60 * 30);
		// 一定要关闭jmx，不然springboot启动会报已经注册了某个jmx的错误
		poolConfig.setJmxEnabled(false);

		log.info("正在初始化BrowserContextObjectPool");
		// 新建一个对象池,传入对象工厂和配置
		return new BrowserContextObjectPool(playwrightFactory, poolConfig).initPool();
	}
}
