/**
 * 
 */
package org.springblade.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.servlet.http.HttpServletRequest;

import org.springblade.config.PlaywrightProperties;
import org.springblade.dto.Result;
import org.springblade.dto.ScreenShotParam;
import org.springblade.dto.ScreenShotParam.Site;
import org.springblade.handler.PlayWrightHandler;
import org.springblade.utils.Kv;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.lang.NonNull;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.servlet.ServletUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@RestController
@RestControllerAdvice
public class PlayWrightController {

	@Autowired
	private PlayWrightHandler playWrightHandler;
	
	@Autowired
	private PlaywrightProperties playwrightProperties;
	
	final ExecutorService threadPool = Executors.newFixedThreadPool(10);
	
	/**
	 * 只允许内网访问
	 * @param request
	 * @param param
	 * @return
	 */
	@PostMapping("/pdf")
	public Result<List<String>> pdf(@NonNull HttpServletRequest request, @RequestBody ScreenShotParam param) {
		String ipAddress = ServletUtil.getClientIP(request);
		log.info("ip地址:{}", ipAddress);
		//开启IP白名单校验
		if(playwrightProperties.isSafeCheckEnabled() 
				&& CollectionUtil.isNotEmpty(playwrightProperties.getWhiteIps())
				&& !playwrightProperties.getWhiteIps().contains(ipAddress)) {
			return Result.failure("无效的请求");
		}
		//校验参数的有效性
		if(StrUtil.isEmpty(param.getToken()) ||
					CollectionUtil.isEmpty(param.getSite())) {
			return Result.failure("必须参数为空，请检查");
		}
		if(param.getSite().stream().anyMatch(f -> StrUtil.isEmpty(f.getUrl()))) {
			return Result.failure("URL不能为空，请检查");
		}
		
		List<Callable<Kv>> tasks = new ArrayList<>();
		for(Site site : param.getSite()) {
			tasks.add(() -> playWrightHandler.screenshot(site, param));
		}
		
		Kv allRes = Kv.create();
		try {//并发执行
			List<Future<Kv>> resList = threadPool.invokeAll(tasks, 10, TimeUnit.MINUTES);
			for(Future<Kv> future : resList) {
				Optional<Kv> optional = Optional.ofNullable(future.get());
				if(optional.isPresent()) {
					allRes.putAll(optional.get());
				}
			}
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return Result.failure("截图失败了");
		}
		List<String> resultList = new ArrayList<String>();
		//按顺序返回截图URL地址
		for(Site site : param.getSite()) {
			Object value = allRes.get(site.getId());
			if(!Objects.isNull(value)) {
				resultList.add(value.toString());
			}
		}
		return Result.success(resultList);
	}

	@ExceptionHandler(Exception.class)
	public Object handleException(Exception ex) {
		return Result.failure("系统错误啦,请稍后重试~", ex.getMessage());
	}
}
