/**
 * 
 */
package org.springblade.handler;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import cn.hutool.core.date.StopWatch;
import org.springblade.config.PlaywrightProperties;
import org.springblade.dto.ScreenShotParam;
import org.springblade.dto.ScreenShotParam.Site;
import org.springblade.pool.BrowserContextObjectPool;
import org.springblade.utils.Kv;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.microsoft.playwright.BrowserContext;
import com.microsoft.playwright.Page;
import com.microsoft.playwright.Page.NavigateOptions;
import com.microsoft.playwright.Page.ScreenshotOptions;
import com.microsoft.playwright.options.ScreenshotScale;
import com.microsoft.playwright.options.ScreenshotType;
import com.microsoft.playwright.options.WaitUntilState;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Component
public class PlayWrightHandler {

	@Autowired
	private BrowserContextObjectPool browserContextPool;

	@Autowired
	private PlaywrightProperties playwrightProperties;

	/**
	 * 截图
	 * 
	 * @param param
	 * @return <id, fileUrl>
	 */
	public Kv screenshot(Site site, ScreenShotParam param) {
		BrowserContext browserContext = null;
		StopWatch stopWatch = new StopWatch("模拟请求页面的耗时情况");
		try {
			stopWatch.start("buildParams");
			// 从池子中获取browserContext
			browserContext = browserContextPool.borrowObject();
			String id = site.getId();
			log.info("[{}]开始请求,domain={},width={},height={},watermark={}", id, playwrightProperties.getWebsiteDomain(),
					site.width(), site.height(), param.getText());
			Map<String, String> headers = new HashMap<>();
			headers.put("Blade-Auth", param.getToken());
			browserContext.setExtraHTTPHeaders(headers);

			long oneTime = System.currentTimeMillis();
			Page page = browserContext.pages().get(0);
			page.setViewportSize(site.width(), site.height());
			String url = String.format("%s?watermark=%s&text=%s",
					playwrightProperties.getWebsiteDomain() + site.getUrl(), param.getWatermark(), param.getText());
			page.onPageError(error -> log.info("[{}]页面出错了|{}", url, error));
			page.onRequestFailed(request -> log.info("[{}]请求地址出错了|method={},error={}", request.url(), request.method(), request.failure()));
			stopWatch.stop();

			stopWatch.start("navigate");
			//跳转到URL，等待页面达到 networkidle 状态： 当没有网络连接活动时，即页面上所有的资源（如图片、脚本等）都已加载完毕，或者在一段时间内没有新的网络请求。
			//最长等待6分钟
			page.navigate(url, new NavigateOptions().setTimeout(360000).setWaitUntil(WaitUntilState.NETWORKIDLE));
			stopWatch.stop();

			stopWatch.start("wait10Secs");
			//等待10s - 最后容忍10s
			page.waitForTimeout(10000);
			stopWatch.stop();

			stopWatch.start("screenshot");
			// 图片存储目录，按月份存储
			String month = DateUtil.format(new Date(), "yyyyMM");
			File dir = FileUtil.file(playwrightProperties.getScreenShotDir(), month);
			if (!dir.exists()) {
				dir.mkdirs();
			}
			String imageName = IdUtil.fastSimpleUUID().toLowerCase() + ".jpg";
			File file = FileUtil.file(dir, imageName);
			// 截图配置
			ScreenshotOptions screenshotOptions = new Page.ScreenshotOptions().setQuality(100)
					.setScale(ScreenshotScale.CSS).setType(ScreenshotType.JPEG).setPath(file.toPath())
					.setFullPage(true);
			// 开始截图
			page.screenshot(screenshotOptions);
			stopWatch.stop();
			// 开始截图
//			byte[] screenshotBytes = page.screenshot(screenshotOptions);
//			waterMark = StrUtil.isBlank(waterMark) ? "保密" : waterMark;
// 			long twoTime = System.currentTimeMillis();
//			try (ByteArrayInputStream av = new ByteArrayInputStream(screenshotBytes);
//					FileOutputStream ao = new FileOutputStream(file);) {
//				//#CCCDCF
//				ImgUtil.pressText(av, ao, waterMark, new Color(204, 205, 207), new Font("宋体", Font.BOLD, 50), 0, 0, 0.4F);
//			} finally {
//				log.info("[{}]截图时间:{}s", id, (System.currentTimeMillis() - twoTime) / 1000.f);
//			}
			log.info("[{}]总耗时:{}s", id, (System.currentTimeMillis() - oneTime) / 1000.f);
			return Kv.create().set(id, playwrightProperties.getScreenShotDomain() + month + "/" + imageName);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return null;
		} finally { // 使用完后，返回池子
			browserContextPool.returnObject(browserContext);
			if(stopWatch.isRunning()) {
				stopWatch.stop();
			}
			//打印最终耗时
			log.info(stopWatch.prettyPrint(TimeUnit.SECONDS));
		}
	}
}
