/**
 * 
 */
package org.springblade.pool;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.pool2.PooledObject;
import org.apache.commons.pool2.PooledObjectFactory;
import org.apache.commons.pool2.impl.DefaultPooledObject;

import com.microsoft.playwright.Browser.NewContextOptions;
import com.microsoft.playwright.BrowserContext;
import com.microsoft.playwright.BrowserType;
import com.microsoft.playwright.Playwright;

/**
 * @author wookvn
 *
 */
public class BrowserContextPooledObjectFactory implements PooledObjectFactory<BrowserContext> {

	private static final Map<BrowserContext, Playwright> PLAYWRIGHT_MAP = new ConcurrentHashMap<>();

	@Override
	public PooledObject<BrowserContext> makeObject() throws Exception {
		Playwright playwright = Playwright.create();
		BrowserContext browserContext = playwright.chromium().launch(new BrowserType.LaunchOptions()
		// .setHeadless(false)
		).newContext(new NewContextOptions().setLocale("zh-CN"));
		browserContext.newPage();
		PLAYWRIGHT_MAP.put(browserContext, playwright);
		return new DefaultPooledObject<BrowserContext>(browserContext);
	}

	@Override
	public void destroyObject(PooledObject<BrowserContext> p) throws Exception {
		BrowserContext browserContext = p.getObject();
		Playwright playwright = PLAYWRIGHT_MAP.remove(browserContext);
		if (playwright != null) {
			playwright.close();
		}
	}

	@Override
	public boolean validateObject(PooledObject<BrowserContext> p) {
		return true;
	}

	@Override
	public void activateObject(PooledObject<BrowserContext> p) throws Exception {
		p.getObject().setExtraHTTPHeaders(new HashMap<>());
		p.getObject().clearCookies();
	}

	@Override
	public void passivateObject(PooledObject<BrowserContext> p) throws Exception {
		//清除header头
		p.getObject().setExtraHTTPHeaders(new HashMap<>());
		p.getObject().clearCookies();
		p.getObject().pages().get(0).evaluate("try {window.localStorage.clear()} catch(e){console.log(e)}");
		p.getObject().pages().get(0).navigate("about:blank");
	}

}
